# Load the data
hght_wide<-read.table('C:/GRE2016/Data/bgs_height_wide_gender.dat', na.string='.')

# Give the variable names
names(hght_wide)<-c('id','female','b_wght','m_age','f_age','dur_lbr',
                         'hght01','hght03','hght06','hght09','hght12', 
                         'hght15','hght18','hght24','hght36')

library(OpenMx)


################# Jenss-Bayley Growth Rate Model #################

# Rate of Change Centered at 1-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.0833, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6708017 0.05001129
#2   sigma2_1      S   b_1i   b_1i   7.3933227 1.35291773
#3  sigma_dy1      S   b_1i   dy_i  -7.8413635 2.32819598
#4  sigma2_dy      S   dy_i   dy_i  26.8941920 5.83415850
#5   sigma_31      S   b_1i   b_3i   4.8680112 1.42261849
#6  sigma_3dy      S   dy_i   b_3i -15.7273517 3.47545920
#7   sigma2_3      S   b_3i   b_3i   9.4399262 2.15190121
#8     beta_1      M      1   b_1i  51.0940356 0.35243289
#9    beta_dy      M      1   dy_i  40.3345305 1.01247621
#10    beta_3      M      1   b_3i -17.8777042 0.46361635
#11         g  gamma      1      1  -2.0630897 0.07796386
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0951               2000.095                       NA
#BIC:      -547.4705               2026.834                 1992.134



# Rate of Change Centered at 3-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.25, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 


#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709178 0.05002753
#2   sigma2_1      S   b_1i   b_1i   7.3891622 1.35169231
#3  sigma_dy1      S   b_1i   dy_i  -5.3705598 1.60722221
#4  sigma2_dy      S   dy_i   dy_i  13.3859797 2.77918533
#5   sigma_31      S   b_1i   b_3i   4.8598350 1.42009656
#6  sigma_3dy      S   dy_i   b_3i -10.9381660 2.39513845
#7   sigma2_3      S   b_3i   b_3i   9.4251581 2.14645975
#8     beta_1      M      1   b_1i  51.0933316 0.35236203
#9    beta_dy      M      1   dy_i  31.2982305 0.55181261
#10    beta_3      M      1   b_3i -17.8806570 0.46359367
#11         g  gamma      1      1  -2.0629726 0.07802445
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134

jb.hght.fit$Load

#mxAlgebra 'Load' 
#$formula:  -gamma * exp(gamma * age_c) * time + (exp(gamma * time) - one) 
#$result:
#             [,1]
# [1,] -0.05528925
# [2,] -0.09501536
# [3,] -0.02766743
# [4,]  0.13662059
# [5,]  0.35878734
# [6,]  0.61551097
# [7,]  0.89286703
# [8,]  1.47957158
# [9,]  2.69718708
#dimnames: NULL

# Rate of Change Centered at 6-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.5, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709151 0.05002817
#2   sigma2_1      S   b_1i   b_1i   7.3886870 1.35230454
#3  sigma_dy1      S   b_1i   dy_i  -2.9577656 0.92649150
#4  sigma2_dy      S   dy_i   dy_i   4.8490861 0.95305398
#5   sigma_31      S   b_1i   b_3i   4.8585337 1.42119841
#6  sigma_3dy      S   dy_i   b_3i  -6.2586657 1.37004265
#7   sigma2_3      S   b_3i   b_3i   9.4224411 2.14793007
#8     beta_1      M      1   b_1i  51.0933191 0.35235024
#9    beta_dy      M      1   dy_i  22.4238933 0.28310948
#10    beta_3      M      1   b_3i -17.8806620 0.46355122
#11         g  gamma      1      1  -2.0629698 0.07802151
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134


# Rate of Change Centered at 9-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.75, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709199 0.05002801
#2   sigma2_1      S   b_1i   b_1i   7.3883864 1.35135824
#3  sigma_dy1      S   b_1i   dy_i  -1.5178766 0.55366637
#4  sigma2_dy      S   dy_i   dy_i   1.9671621 0.37415791
#5   sigma_31      S   b_1i   b_3i   4.8579880 1.41923981
#6  sigma_3dy      S   dy_i   b_3i  -3.4662424 0.79330119
#7   sigma2_3      S   b_3i   b_3i   9.4214473 2.14518760
#8     beta_1      M      1   b_1i  51.0932875 0.35234520
#9    beta_dy      M      1   dy_i  17.1253997 0.21767355
#10    beta_3      M      1   b_3i -17.8806575 0.46353518
#11         g  gamma      1      1  -2.0629780 0.07802107
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty
#AIC:       838.0950               2000.095
#BIC:      -547.4706               2026.834


# Rate of Change Centered at 12-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709176 0.05002767
#2   sigma2_1      S   b_1i   b_1i   7.3888049 1.35172144
#3  sigma_dy1      S   b_1i   dy_i  -0.6584699 0.38304345
#4  sigma2_dy      S   dy_i   dy_i   1.0355629 0.20072879
#5   sigma_31      S   b_1i   b_3i   4.8585532 1.41972205
#6  sigma_3dy      S   dy_i   b_3i  -1.7994623 0.51047055
#7   sigma2_3      S   b_3i   b_3i   9.4221856 2.14580967
#8     beta_1      M      1   b_1i  51.0933116 0.35235206
#9    beta_dy      M      1   dy_i  13.9619118 0.17609211
#10    beta_3      M      1   b_3i -17.8806678 0.46354728
#11         g  gamma      1      1  -2.0629711 0.07802064
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134


# Rate of Change Centered at 15-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1.25, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709180 0.05002819
#2   sigma2_1      S   b_1i   b_1i   7.3883986 1.35150668
#3  sigma_dy1      S   b_1i   dy_i  -0.1452093 0.33278990
#4  sigma2_dy      S   dy_i   dy_i   0.7605279 0.15746982
#5   sigma_31      S   b_1i   b_3i   4.8580296 1.41964696
#6  sigma_3dy      S   dy_i   b_3i  -0.8041239 0.41685602
#7   sigma2_3      S   b_3i   b_3i   9.4216022 2.14594681
#8     beta_1      M      1   b_1i  51.0933054 0.35234578
#9    beta_dy      M      1   dy_i  12.0731155 0.13905436
#10    beta_3      M      1   b_3i -17.8806568 0.46354073
#11         g  gamma      1      1  -2.0629742 0.07802278
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134


# Rate of Change Centered at 18-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1.5, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709189 0.05002798
#2   sigma2_1      S   b_1i   b_1i   7.3885541 1.35183298
#3  sigma_dy1      S   b_1i   dy_i   0.1611777 0.33243290
#4  sigma2_dy      S   dy_i   dy_i   0.6965753 0.15409443
#5   sigma_31      S   b_1i   b_3i   4.8582611 1.42016995
#6  sigma_3dy      S   dy_i   b_3i  -0.2099161 0.41299106
#7   sigma2_3      S   b_3i   b_3i   9.4219009 2.14651060
#8     beta_1      M      1   b_1i  51.0932990 0.35234828
#9    beta_dy      M      1   dy_i  10.9454023 0.11851896
#10    beta_3      M      1   b_3i -17.8806688 0.46354406
#11         g  gamma      1      1  -2.0629739 0.07802185
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134

# Rate of Change Centered at 24-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 2, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709188 0.05002807
#2   sigma2_1      S   b_1i   b_1i   7.3886285 1.35182919
#3  sigma_dy1      S   b_1i   dy_i   0.4533724 0.35413776
#4  sigma2_dy      S   dy_i   dy_i   0.7054081 0.16701307
#5   sigma_31      S   b_1i   b_3i   4.8583914 1.42012994
#6  sigma_3dy      S   dy_i   b_3i   0.3567367 0.45117825
#7   sigma2_3      S   b_3i   b_3i   9.4220550 2.14652823
#8     beta_1      M      1   b_1i  51.0933082 0.35235144
#9    beta_dy      M      1   dy_i   9.8700929 0.12667829
#10    beta_3      M      1   b_3i -17.8806586 0.46354012
#11         g  gamma      1      1  -2.0629722 0.07802087
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134


# Rate of Change Centered at 36-months 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 15, -18),
 labels=c('beta_1','beta_dy','beta_3') ),

mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 3, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix    row    col    Estimate  Std.Error
#1   sigma2_u      S hght01 hght01   0.6709180 0.05002751
#2   sigma2_1      S   b_1i   b_1i   7.3886253 1.35166790
#3  sigma_dy1      S   b_1i   dy_i   0.5946397 0.37173447
#4  sigma2_dy      S   dy_i   dy_i   0.7341170 0.17906950
#5   sigma_31      S   b_1i   b_3i   4.8583592 1.41977730
#6  sigma_3dy      S   dy_i   b_3i   0.6307069 0.48355640
#7   sigma2_3      S   b_3i   b_3i   9.4220079 2.14608255
#8     beta_1      M      1   b_1i  51.0933030 0.35234910
#9    beta_dy      M      1   dy_i   9.3501194 0.16130057
#10    beta_3      M      1   b_3i -17.8806681 0.46354638
#11         g  gamma      1      1  -2.0629719 0.07802196
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.095 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.0950               2000.095                       NA
#BIC:      -547.4706               2026.834                 1992.134


#### Plot of Rate of Change ####

time=c(1,3,6,9,12,15,18,24,36)
beta_dy=c(40.33, 31.30, 22.42, 17.13, 13.96, 12.07, 10.95, 9.87, 9.35)
sigma2_dy=c(26.89,13.39, 4.84, 1.97, 1.04, 0.76, 0.70, 0.71, 0.73)
upper=beta_dy + 1.96*sqrt(sigma2_dy)
lower=beta_dy - 1.96*sqrt(sigma2_dy)

plot1a = cbind(time,beta_dy,1)
plot1b = cbind(time,upper,2)
plot1c = cbind(time,lower,3)

plot1d = rbind(plot1a,plot1b,plot1c)

plot1e = data.frame(plot1d)

plot1e

library(ggplot2)

plot_rate <- ggplot(data=plot1e, aes(x=time, y=beta_dy, id=V3)) +
                   geom_line() +
                   theme_bw() +
                   scale_x_continuous(breaks = c(0,10,20,30,40), name = "Chronological Age") + 
                   scale_y_continuous(breaks = c(0,10,20,30,40,50,60), name = "Rate of Change (cm/year)")

print(plot_rate)



################# Including Labor Duration as Covariate #################

summary(hght_wide$dur_lbr)

hght_wide$dur_lbr = (hght_wide$dur_lbr - 9.725)/7.309;

# Rate of Change Centered at 1-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 40, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.0833, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03811044 0.38621777
#2    beta_12      A    dy_i dur_lbr  -2.22149041 0.73334917
#3    beta_13      A    b_3i dur_lbr   1.50799552 0.43893127
#4   sigma2_u      S  hght01  hght01   0.67091229 0.04998513
#5   sigma2_x      S dur_lbr dur_lbr   1.04218672 0.20916333
#6   sigma2_1      S    b_1i    b_1i   6.23772814 1.31638762
#7  sigma_dy1      S    b_1i    dy_i  -5.42855793 2.22817567
#8  sigma2_dy      S    dy_i    dy_i  22.04265792 5.36033872
#9   sigma_31      S    b_1i    b_3i   3.25306913 1.36463193
#10 sigma_3dy      S    dy_i    b_3i -12.50500697 3.18814250
#11  sigma2_3      S    b_3i    b_3i   7.27999923 1.95661374
#12     tau_x      M       1 dur_lbr   0.03115427 0.13208354
#13   beta_01      M       1    b_1i  51.10326539 0.33728542
#14   beta_02      M       1    dy_i  40.24930916 0.98385756
#15   beta_03      M       1    b_3i -17.93023723 0.44372795
#16         g  gamma       1       1  -2.05164432 0.07741290
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0416               2156.042                       NA
#BIC:      -627.4956               2194.935                 2144.462


# Rate of Change Centered at 3-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.25, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03869978 0.38677292
#2    beta_12      A    dy_i dur_lbr  -1.46725574 0.51727894
#3    beta_13      A    b_3i dur_lbr   1.50908763 0.43957551
#4   sigma2_u      S  hght01  hght01   0.67093023 0.04998782
#5   sigma2_x      S dur_lbr dur_lbr   1.04236221 0.20926260
#6   sigma2_1      S    b_1i    b_1i   6.23557131 1.31633374
#7  sigma_dy1      S    b_1i    dy_i  -3.79411256 1.54690502
#8  sigma2_dy      S    dy_i    dy_i  11.32496136 2.59655390
#9   sigma_31      S    b_1i    b_3i   3.24913674 1.36372412
#10 sigma_3dy      S    dy_i    b_3i  -8.84404416 2.21365938
#11  sigma2_3      S    b_3i    b_3i   7.26744160 1.95391859
#12     tau_x      M       1 dur_lbr   0.03136501 0.13208106
#13   beta_01      M       1    b_1i  51.10276945 0.33723347
#14   beta_02      M       1    dy_i  31.26848005 0.53283860
#15   beta_03      M       1    b_3i -17.93039685 0.44356062
#16         g  gamma       1       1  -2.05174749 0.07741085
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462


# Rate of Change Centered at 6-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.5, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03880713 0.38603123
#2    beta_12      A    dy_i dur_lbr  -0.72340434 0.30913070
#3    beta_13      A    b_3i dur_lbr   1.50913533 0.43851319
#4   sigma2_u      S  hght01  hght01   0.67092183 0.04998616
#5   sigma2_x      S dur_lbr dur_lbr   1.04241252 0.20923545
#6   sigma2_1      S    b_1i    b_1i   6.23502086 1.31533854
#7  sigma_dy1      S    b_1i    dy_i  -2.19201853 0.89382330
#8  sigma2_dy      S    dy_i    dy_i   4.37136955 0.90972703
#9   sigma_31      S    b_1i    b_3i   3.24839733 1.36179374
#10 sigma_3dy      S    dy_i    b_3i  -5.26103820 1.27838972
#11  sigma2_3      S    b_3i    b_3i   7.26644946 1.95135586
#12     tau_x      M       1 dur_lbr   0.03137122 0.13206298
#13   beta_01      M       1    b_1i  51.10272603 0.33722241
#14   beta_02      M       1    dy_i  22.42997998 0.27534817
#15   beta_03      M       1    b_3i -17.93042760 0.44354572
#16         g  gamma       1       1  -2.05176383 0.07741160
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462


# Rate of Change Centered at 9-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 0.75, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03881256 0.38600659
#2    beta_12      A    dy_i dur_lbr  -0.27800653 0.19478288
#3    beta_13      A    b_3i dur_lbr   1.50915504 0.43847551
#4   sigma2_u      S  hght01  hght01   0.67092190 0.04998533
#5   sigma2_x      S dur_lbr dur_lbr   1.04241232 0.20922128
#6   sigma2_1      S    b_1i    b_1i   6.23494530 1.31338730
#7  sigma_dy1      S    b_1i    dy_i  -1.23321771 0.53161589
#8  sigma2_dy      S    dy_i    dy_i   1.89878561 0.36497977
#9   sigma_31      S    b_1i    b_3i   3.24822165 1.35849245
#10 sigma_3dy      S    dy_i    b_3i  -3.11625885 0.74943889
#11  sigma2_3      S    b_3i    b_3i   7.26604600 1.94801898
#12     tau_x      M       1 dur_lbr   0.03136874 0.13207848
#13   beta_01      M       1    b_1i  51.10272623 0.33722048
#14   beta_02      M       1    dy_i  17.13800531 0.21600639
#15   beta_03      M       1    b_3i -17.93042042 0.44353585
#16         g  gamma       1       1  -2.05176585 0.07740938
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462


# Rate of Change Centered at 12-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03882169 0.38680898
#2    beta_12      A    dy_i dur_lbr  -0.01132600 0.13910515
#3    beta_13      A    b_3i dur_lbr   1.50915958 0.43944104
#4   sigma2_u      S  hght01  hght01   0.67092237 0.04998661
#5   sigma2_x      S dur_lbr dur_lbr   1.04241569 0.20930661
#6   sigma2_1      S    b_1i    b_1i   6.23489452 1.31641400
#7  sigma_dy1      S    b_1i    dy_i  -0.65920927 0.36113380
#8  sigma2_dy      S    dy_i    dy_i   1.02432620 0.19735948
#9   sigma_31      S    b_1i    b_3i   3.24814371 1.36346530
#10 sigma_3dy      S    dy_i    b_3i  -1.83225420 0.48299367
#11  sigma2_3      S    b_3i    b_3i   7.26592065 1.95311078
#12     tau_x      M       1 dur_lbr   0.03136766 0.13209058
#13   beta_01      M       1    b_1i  51.10272068 0.33722368
#14   beta_02      M       1    dy_i  13.96953005 0.17558330
#15   beta_03      M       1    b_3i -17.93041632 0.44354323
#16         g  gamma       1       1  -2.05176768 0.07741329
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462



# Rate of Change Centered at 15-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 10, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1.25, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col    Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.0388434 0.38653878
#2    beta_12      A    dy_i dur_lbr   0.1483390 0.11759584
#3    beta_13      A    b_3i dur_lbr   1.5091868 0.43910027
#4   sigma2_u      S  hght01  hght01   0.6709195 0.04998558
#5   sigma2_x      S dur_lbr dur_lbr   1.0424127 0.20924611
#6   sigma2_1      S    b_1i    b_1i   6.2348792 1.31587295
#7  sigma_dy1      S    b_1i    dy_i  -0.3155616 0.30565695
#8  sigma2_dy      S    dy_i    dy_i   0.7179554 0.15069556
#9   sigma_31      S    b_1i    b_3i   3.2481340 1.36261381
#10 sigma_3dy      S    dy_i    b_3i  -1.0635142 0.38278102
#11  sigma2_3      S    b_3i    b_3i   7.2659379 1.95248781
#12     tau_x      M       1 dur_lbr   0.0313666 0.13208411
#13   beta_01      M       1    b_1i  51.1027192 0.33722334
#14   beta_02      M       1    dy_i  12.0724669 0.13733303
#15   beta_03      M       1    b_3i -17.9304148 0.44354046
#16         g  gamma       1       1  -2.0517657 0.07741320
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462


# Rate of Change Centered at 18-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1.5, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col    Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.0388187 0.38617696
#2    beta_12      A    dy_i dur_lbr   0.2439476 0.11225240
#3    beta_13      A    b_3i dur_lbr   1.5091839 0.43882765
#4   sigma2_u      S  hght01  hght01   0.6709216 0.04998633
#5   sigma2_x      S dur_lbr dur_lbr   1.0424074 0.20924944
#6   sigma2_1      S    b_1i    b_1i   6.2349556 1.31540405
#7  sigma_dy1      S    b_1i    dy_i  -0.1097992 0.30180231
#8  sigma2_dy      S    dy_i    dy_i   0.6123680 0.14220485
#9   sigma_31      S    b_1i    b_3i   3.2482468 1.36165659
#10 sigma_3dy      S    dy_i    b_3i  -0.6032403 0.36739079
#11  sigma2_3      S    b_3i    b_3i   7.2660632 1.95157486
#12     tau_x      M       1 dur_lbr   0.0313659 0.13207000
#13   beta_01      M       1    b_1i  51.1027344 0.33722234
#14   beta_02      M       1    dy_i  10.9366328 0.11456420
#15   beta_03      M       1    b_3i -17.9304019 0.44353670
#16         g  gamma       1       1  -2.0517663 0.07741112
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462


# Rate of Change Centered at 24-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 2, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03882965 0.38653722
#2    beta_12      A    dy_i dur_lbr   0.33545645 0.11373099
#3    beta_13      A    b_3i dur_lbr   1.50917869 0.43909122
#4   sigma2_u      S  hght01  hght01   0.67092201 0.04998675
#5   sigma2_x      S dur_lbr dur_lbr   1.04241490 0.20924362
#6   sigma2_1      S    b_1i    b_1i   6.23493772 1.31530871
#7  sigma_dy1      S    b_1i    dy_i   0.08716922 0.32063157
#8  sigma2_dy      S    dy_i    dy_i   0.56592789 0.14830642
#9   sigma_31      S    b_1i    b_3i   3.24822067 1.36175519
#10 sigma_3dy      S    dy_i    b_3i  -0.16264938 0.39151117
#11  sigma2_3      S    b_3i    b_3i   7.26605037 1.95175447
#12     tau_x      M       1 dur_lbr   0.03137006 0.13207177
#13   beta_01      M       1    b_1i  51.10272301 0.33722179
#14   beta_02      M       1    dy_i   9.84939868 0.12048200
#15   beta_03      M       1    b_3i -17.93042512 0.44354266
#16         g  gamma       1       1  -2.05176592 0.07741255
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462




# Rate of Change Centered at 36-month 

jb.hght.omx <- mxModel("Jenss Bayley Growth Rate Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','dy_i','b_3i'),

# residual variances  
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent variable covariances 
mxPath(from=c('b_1i','dy_i','b_3i'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(10, 0, 0,
                                     1, 0,
                                      10),
 labels=c('sigma2_1','sigma_dy1','sigma_31','sigma2_dy','sigma_3dy','sigma2_3')),

# factor loadings 
mxPath(from='b_1i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='dy_i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=c(.0833, .25, .5, .75, 1, 1.25, 1.5, 2, 3)),

mxPath(from='b_3i', to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]','Load[2,1]','Load[3,1]','Load[4,1]','Load[5,1]','Load[6,1]','Load[7,1]','Load[8,1]','Load[9,1]')),

# latent variable means  
mxPath(from='one', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 30, -18),
 labels=c('beta_01','beta_02','beta_03') ),

# adding covariate
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x'),

mxPath(from='dur_lbr', to=c('b_1i','dy_i','b_3i'),
 arrows=1, free=TRUE, values=0, labels=c('beta_11','beta_12','beta_13')),

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE, values=0, labels='tau_x'),

# nonlinear constraint
mxMatrix('Full', 9, 1, free = TRUE, values = -2, labels = 'g', name = 'gamma'),
mxMatrix('Full', 9, 1, free = FALSE, values = c(.0833,.25,.5,.75,1,1.25,1.5,2,3), name = 'time'),
mxMatrix('Full', 9, 1, free = FALSE, values = 3, name = 'age_c'),
mxMatrix('Full', 9, 1, free = FALSE, values = 1, name = 'one'),
mxAlgebra(-gamma*exp(gamma*age_c)*time+(exp(gamma*time)-one), name='Load')

) # Close Model

jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit) 

#free parameters:
#        name matrix     row     col     Estimate  Std.Error
#1    beta_11      A    b_1i dur_lbr   1.03882591 0.38604109
#2    beta_12      A    dy_i dur_lbr   0.38002245 0.11698671
#3    beta_13      A    b_3i dur_lbr   1.50917301 0.43872858
#4   sigma2_u      S  hght01  hght01   0.67092197 0.04998667
#5   sigma2_x      S dur_lbr dur_lbr   1.04240861 0.20923190
#6   sigma2_1      S    b_1i    b_1i   6.23497035 1.31528232
#7  sigma_dy1      S    b_1i    dy_i   0.18308629 0.33715010
#8  sigma2_dy      S    dy_i    dy_i   0.56265949 0.15648390
#9   sigma_31      S    b_1i    b_3i   3.24826119 1.36188159
#10 sigma_3dy      S    dy_i    b_3i   0.05191395 0.41663258
#11  sigma2_3      S    b_3i    b_3i   7.26607433 1.95209059
#12     tau_x      M       1 dur_lbr   0.03136760 0.13208108
#13   beta_01      M       1    b_1i  51.10271829 0.33722485
#14   beta_02      M       1    dy_i   9.31993265 0.15573365
#15   beta_03      M       1    b_3i -17.93041749 0.44354138
#16         g  gamma       1       1  -2.05176824 0.07741326
#
#observed statistics:  637 
#estimated parameters:  16 
#degrees of freedom:  621 
#-2 log likelihood:  2124.042 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       882.0415               2156.042                       NA
#BIC:      -627.4957               2194.935                 2144.462

#### Plot of Rate of Association between Labor Duration & Rate of Growth ####


time=c(1,3,6,9,12,15,18,24,36)
beta_12=c(-2.221, -1.467, -0.723, -0.278, -0.011, 0.148, 0.244, 0.335, 0.380)
se_beta_12=c(0.733, 0.517, 0.309, 0.195, 0.139, 0.118, 0.112, 0.114, 0.117)

upper=beta_12 + 1.96*se_beta_12
lower=beta_12 - 1.96*se_beta_12

plot1a = cbind(time,beta_12,1)
plot1b = cbind(time,upper,2)
plot1c = cbind(time,lower,3)

plot1d = rbind(plot1a,plot1b,plot1c)

plot1e = data.frame(plot1d)

plot1e

plot_eff <- ggplot(data=plot1e, aes(x=time, y=beta_12, id=V3)) +
                   geom_line() +
                   theme_bw() +
                   scale_x_continuous(breaks = c(0,10,20,30,40), name = "Chronological Age") + 
                   scale_y_continuous(breaks = c(-3,-2.5,-2.0,-1.5,-1.0,-0.5,0,0.5,1), name = "Effect on Changes in Height")

print(plot_eff)



############################################################################
############################################################################
############################################################################
############################################################################
############################################################################



################# Jenss-Bayley Latent Change Model ################# 

jb.hght.omx <- mxModel("Jenss Bayley Latent Change Model, Path Specification", type="RAM",
mxData(observed=hght_wide, type="raw"),
manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
latentVars=c('b_1i','b_2i','b_3i',               
             'ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
             'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
             'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
             'ly30','ly31','ly32','ly33','ly34','ly35','ly36',
                    'dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09',
             'dy10','dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19',
             'dy20','dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29',
             'dy30','dy31','dy32','dy33','dy34','dy35','dy36'),

# Residual Variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# Latent True Scores 
mxPath(from=c('ly01','ly03', 'ly06','ly09','ly12','ly15','ly18','ly24','ly36'),
 to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Paths 
mxPath(from=c('ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
              'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
              'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
              'ly30','ly31','ly32','ly33','ly34','ly35'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1),

# Latent Change Scores
mxPath(from=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
              'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
              'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
              'dy31','dy32','dy33','dy34','dy35','dy36'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1 ),

# Factor Loadings 
mxPath(from='b_1i', to='ly00', arrows=1, free=FALSE, values=1),

mxPath(from='b_2i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='b_3i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]', 'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]',
          'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]', 'Load[10,1]',
          'Load[11,1]','Load[12,1]','Load[13,1]','Load[14,1]','Load[15,1]',
          'Load[16,1]','Load[17,1]','Load[18,1]','Load[19,1]','Load[20,1]',
          'Load[21,1]','Load[22,1]','Load[23,1]','Load[24,1]','Load[25,1]',
          'Load[26,1]','Load[27,1]','Load[28,1]','Load[29,1]','Load[30,1]',
          'Load[31,1]','Load[32,1]','Load[33,1]','Load[34,1]','Load[35,1]',
          'Load[36,1]')),

# Latent Variable Covariances 
mxPath(from=c('b_1i','b_2i','b_3i'), connect='unique.pairs', arrows=2,
 free=TRUE, values = c(8,0,0,.1,0,12),
 labels=c('sigma2_1','sigma_21','sigma_31','sigma2_2','sigma_32','sigma2_3')),

# Latent Variable Means  
mxPath(from='one', to=c('b_1i','b_2i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 1, -20), labels=c('beta_1','beta_2','beta_3')),

mxMatrix('Full', 36, 1, free = TRUE, values = -1, labels = 'g', name = 'gamma'),
mxMatrix('Full', 36, 1, free = FALSE, values = c( 1, 2, 3, 4, 5, 6, 7, 8, 9,10,
                                                 11,12,13,14,15,16,17,18,19,20,
                                                 21,22,23,24,25,26,27,28,29,30,
                                                 31,32,33,34,35,36), name = 'time'),
mxAlgebra(gamma*exp(gamma*time), name='Load')

) # Close Model



jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit)

#free parameters:
#       name matrix    row    col      Estimate   Std.Error
#1  sigma2_u      S hght01 hght01   0.670980400 0.050032497
#2  sigma2_1      S   b_1i   b_1i   7.389475093 1.351926441
#3  sigma_21      S   b_1i   b_2i   0.051259168 0.031235912
#4  sigma2_2      S   b_2i   b_2i   0.005135751 0.001258138
#5  sigma_31      S   b_1i   b_3i   5.302019879 1.549272128
#6  sigma_32      S   b_2i   b_3i   0.060944216 0.044478289
#7  sigma2_3      S   b_3i   b_3i  11.217447404 2.548071659
#8    beta_1      M      1   b_1i  51.091521148 0.352396075
#9    beta_2      M      1   b_2i   0.772904804 0.014355921
#10   beta_3      M      1   b_3i -19.510558876 0.481331366
#11        g  gamma      1      1  -0.171949628 0.006503177
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.128 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.1283               2000.128                       NA
#BIC:      -547.4373               2026.867                 1992.168



jb.hght.fit$output

## Model Expectations ##
jb.hght.fit$output$expectations
jb.hght.fit$output$algebras

attr(jb.hght.fit$output$expectations[[1]], which="UnfilteredExpCov")




## Mean and Variance Expectations of Change Scores ##

jb.hght.omx <- mxModel("Jenss Bayley Latent Change Model, Path Specification", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 latentVars=c('b_1i','b_2i','b_3i',               
              'ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
              'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
              'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
              'ly30','ly31','ly32','ly33','ly34','ly35','ly36',
                     'dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09',
              'dy10','dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19',
              'dy20','dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29',
              'dy30','dy31','dy32','dy33','dy34','dy35','dy36'), 

# Residual Variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# Latent True Scores 
mxPath(from=c('ly01','ly03', 'ly06','ly09','ly12','ly15','ly18','ly24','ly36'),
 to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Paths 
mxPath(from=c('ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
              'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
              'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
              'ly30','ly31','ly32','ly33','ly34','ly35'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1),

# Latent Change Scores
mxPath(from=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
              'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
              'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
              'dy31','dy32','dy33','dy34','dy35','dy36'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1 ),

# Factor Loadings 
mxPath(from='b_1i', to='ly00', arrows=1, free=FALSE, values=1),

mxPath(from='b_2i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='b_3i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]', 'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]',
          'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]', 'Load[10,1]',
          'Load[11,1]','Load[12,1]','Load[13,1]','Load[14,1]','Load[15,1]',
          'Load[16,1]','Load[17,1]','Load[18,1]','Load[19,1]','Load[20,1]',
          'Load[21,1]','Load[22,1]','Load[23,1]','Load[24,1]','Load[25,1]',
          'Load[26,1]','Load[27,1]','Load[28,1]','Load[29,1]','Load[30,1]',
          'Load[31,1]','Load[32,1]','Load[33,1]','Load[34,1]','Load[35,1]',
          'Load[36,1]')),

# Latent Variable Covariances 
mxPath(from=c('b_1i','b_2i','b_3i'), connect='unique.pairs', arrows=2,
 free=TRUE, values = c(8,0,0,.1,0,12),
 labels=c('sigma2_1','sigma_21','sigma_31','sigma2_2','sigma_32','sigma2_3')),

# Latent Variable Means  
mxPath(from='one', to=c('b_1i','b_2i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 1, -20), labels=c('beta_1','beta_2','beta_3')),

mxMatrix('Full', 36, 1, free = TRUE, values = -1, labels = 'g', name = 'gamma'),
mxMatrix('Full', 36, 1, free = FALSE, values = c( 1, 2, 3, 4, 5, 6, 7, 8, 9,10,
                                                 11,12,13,14,15,16,17,18,19,20,
                                                 21,22,23,24,25,26,27,28,29,30,
                                                 31,32,33,34,35,36), name = 'time'),
mxAlgebra(gamma*exp(gamma*time), name='Load'),


# Mean & Variance Expectations of Changes
mxMatrix('Full', 36, 1, free = TRUE,  values =   1, labels =   'beta_2', name = 'bet_2'),
mxMatrix('Full', 36, 1, free = TRUE,  values = -20, labels =   'beta_3', name = 'bet_3'),
mxMatrix('Full', 36, 1, free = TRUE,  values =  .1, labels = 'sigma2_2', name = 'sig2_2'),
mxMatrix('Full', 36, 1, free = TRUE,  values =  12, labels = 'sigma2_3', name = 'sig2_3'),
mxMatrix('Full', 36, 1, free = TRUE,  values =   0, labels = 'sigma_32', name = 'sig_32'),

mxMatrix('Full', 36, 1, free = FALSE, 
    labels = c(             'Load[1,1]' ,'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]', 'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]',
               'Load[10,1]','Load[11,1]','Load[12,1]','Load[13,1]','Load[14,1]','Load[15,1]','Load[16,1]','Load[17,1]','Load[18,1]','Load[19,1]',
               'Load[20,1]','Load[21,1]','Load[22,1]','Load[23,1]','Load[24,1]','Load[25,1]','Load[26,1]','Load[27,1]','Load[28,1]','Load[29,1]',
               'Load[30,1]','Load[31,1]','Load[32,1]','Load[33,1]','Load[34,1]','Load[35,1]','Load[36,1]'),
    name = 'b3_load'),

mxAlgebra(bet_2+bet_3*b3_load, name='Means'),
mxAlgebra(sig2_2+b3_load*sig2_3*b3_load+sig_32*b3_load+sig_32*b3_load, name='Variances')

) # Close Model
 
jb.hght.fit <- mxRun(jb.hght.omx)

summary(jb.hght.fit)

#Summary of Jenss Bayley Latent Change Model, Path Specification 
# 
#free parameters:
#       name matrix    row    col      Estimate   Std.Error
#1  sigma2_u      S hght01 hght01   0.670980400 0.050032497
#2  sigma2_1      S   b_1i   b_1i   7.389475093 1.351926441
#3  sigma_21      S   b_1i   b_2i   0.051259168 0.031235912
#4  sigma2_2      S   b_2i   b_2i   0.005135751 0.001258138
#5  sigma_31      S   b_1i   b_3i   5.302019879 1.549272128
#6  sigma_32      S   b_2i   b_3i   0.060944216 0.044478289
#7  sigma2_3      S   b_3i   b_3i  11.217447404 2.548071659
#8    beta_1      M      1   b_1i  51.091521148 0.352396075
#9    beta_2      M      1   b_2i   0.772904804 0.014355921
#10   beta_3      M      1   b_3i -19.510558876 0.481331366
#11        g  gamma      1      1  -0.171949628 0.006503177
#
#observed statistics:  581 
#estimated parameters:  11 
#degrees of freedom:  570 
#-2 log likelihood:  1978.128 
#number of observations:  84 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:       838.1283               2000.128                       NA
#BIC:      -547.4373               2026.867                 1992.168
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-12-09 21:51:56 
#Wall clock time (HH:MM:SS.hh): 00:00:08.11 
#optimizer:  NPSOL 
#OpenMx version number: 2.0.0.4004 
#Need help?  See help(mxSummary) 


jb.hght.fit$Means

#mxAlgebra 'Means' 
#$formula:  bet_2 + bet_3 * b3_load 
#$result:
#           [,1]
# [1,] 3.5977469
# [2,] 3.1514828
# [3,] 2.7757188
# [4,] 2.4593175
# [5,] 2.1929007
# [6,] 1.9685720
# [7,] 1.7796824
# [8,] 1.6206332
# [9,] 1.4867104
#[10,] 1.3739446
#[11,] 1.2789933
#[12,] 1.1990422
#[13,] 1.1317217
#[14,] 1.0750364
#[15,] 1.0273061
#[16,] 0.9871162
#[17,] 0.9532754
#[18,] 0.9247808
#[19,] 0.9007876
#[20,] 0.8805849
#[21,] 0.8635738
#[22,] 0.8492500
#[23,] 0.8371891
#[24,] 0.8270336
#[25,] 0.8184824
#[26,] 0.8112821
#[27,] 0.8052193
#[28,] 0.8001143
#[29,] 0.7958158
#[30,] 0.7921964
#[31,] 0.7891487
#[32,] 0.7865825
#[33,] 0.7844218
#[34,] 0.7826023
#[35,] 0.7810703
#[36,] 0.7797803


jb.hght.fit$Variances

#mxAlgebra 'Variances' 
#$formula:  sig2_2 + b3_load * sig2_3 * b3_load + sig_32 * b3_load + sig_32 * b3_load 
#$result:
#             [,1]
# [1,] 0.222637011
# [2,] 0.156996708
# [3,] 0.110828567
# [4,] 0.078407607
# [5,] 0.055684073
# [6,] 0.039794478
# [7,] 0.028715172
# [8,] 0.021016912
# [9,] 0.015691017
#[10,] 0.012026249
#[11,] 0.009521638
#[12,] 0.007824774
#[13,] 0.006688139
#[14,] 0.005938209
#[15,] 0.005453615
#[16,] 0.005149703
#[17,] 0.004967628
#[18,] 0.004866660
#[19,] 0.004818753
#[20,] 0.004804725
#[21,] 0.004811569
#[22,] 0.004830558
#[23,] 0.004855924
#[24,] 0.004883932
#[25,] 0.004912229
#[26,] 0.004939398
#[27,] 0.004964644
#[28,] 0.004987582
#[29,] 0.005008087
#[30,] 0.005026198
#[31,] 0.005042046
#[32,] 0.005055815
#[33,] 0.005067710
#[34,] 0.005077939
#[35,] 0.005086704
#[36,] 0.005094191
#dimnames: NULL


# Alternatively, you can use the RAM Matrices output from OpenMx to 
# Calculate Model Expectations.

# The benefit of this approach is that the only thing that needs to change
# are the some of the matrix dimensions.

# First, Calculate the Full Asymmetric Matrix by appending the Mean Vector to the A Matrix
# Second, Add a row of 0s to make the matrix square

Full_A = rbind(cbind(jb.hght.fit$matrices$A$values, t(jb.hght.fit$matrices$M$values)),0)
Full_A

# Next, Create the Full Symmetric Matrix by adding a row and column of 0s and a 1 as the 
# last element

Full_S = rbind(cbind(jb.hght.fit$matrices$S$values,0),0)
Full_S[86,86]=1
Full_S

# Create identity matrix
ident = diag(86)
ident

# Effects Matrix is the inverse of the Identity Matrix minus the A matrix
effects = solve(ident - Full_A)

# Total Variable Expectations is the Effects Matrix times the S matrix times the
# transpose of the Effects Matrix
Total_Variable_Expectations = effects %*% Full_S %*% t(effects)

# Mean vector is the last column of the Total Variable Expectations
Mean_Exp = Total_Variable_Expectations[,86]

# Calculate the Square of the Mean Vector
Mean_Exp2 = as.matrix(Total_Variable_Expectations[,86]) %*% t(as.matrix(Total_Variable_Expectations[,86]))

# Covariance expectations are the Total Variable Expectations minus the Square of the Mean Vector
Cov_Exp = Total_Variable_Expectations - Mean_Exp2

# Just select diagonal elements of the Covariance Expectations
Var_Expect = ident * Cov_Exp
Var_Expect




################# Including Labor Duration & Calculating Indirect Effect ################# 

jb.hght.omx <- mxModel("Jenss Bayley Latent Change Model", type="RAM",
 mxData(observed=hght_wide, type="raw"),
 manifestVars=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36','dur_lbr'),
 latentVars=c('b_1i','b_2i','b_3i',               
              'ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
              'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
              'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
              'ly30','ly31','ly32','ly33','ly34','ly35','ly36',
                     'dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09',
              'dy10','dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19',
              'dy20','dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29',
              'dy30','dy31','dy32','dy33','dy34','dy35','dy36'), 

# residual variances
mxPath(from=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=2, free=TRUE, values=2, labels='sigma2_u'),

# latent true scores 
mxPath(from=c('ly01','ly03', 'ly06','ly09','ly12','ly15','ly18','ly24','ly36'),
 to=c('hght01','hght03','hght06','hght09','hght12','hght15','hght18','hght24','hght36'),
 arrows=1, free=FALSE, values=1),

# Autoregressive Paths 
mxPath(from=c('ly00','ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09',
              'ly10','ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19',
              'ly20','ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29',
              'ly30','ly31','ly32','ly33','ly34','ly35'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1),

# Latent Change Scores
mxPath(from=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
              'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
              'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
              'dy31','dy32','dy33','dy34','dy35','dy36'),
         to=c('ly01','ly02','ly03','ly04','ly05','ly06','ly07','ly08','ly09','ly10',
              'ly11','ly12','ly13','ly14','ly15','ly16','ly17','ly18','ly19','ly20',
              'ly21','ly22','ly23','ly24','ly25','ly26','ly27','ly28','ly29','ly30',
              'ly31','ly32','ly33','ly34','ly35','ly36'),
 arrows=1, free=FALSE, values=1),

# factor loadings 
mxPath(from='b_1i', to='ly00', arrows=1, free=FALSE, values=1),

mxPath(from='b_2i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE, values=1),

mxPath(from='b_3i',
 to=c('dy01','dy02','dy03','dy04','dy05','dy06','dy07','dy08','dy09','dy10',
      'dy11','dy12','dy13','dy14','dy15','dy16','dy17','dy18','dy19','dy20',
      'dy21','dy22','dy23','dy24','dy25','dy26','dy27','dy28','dy29','dy30',
      'dy31','dy32','dy33','dy34','dy35','dy36'),
 arrows=1, free=FALSE,
 labels=c('Load[1,1]', 'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]',
          'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]', 'Load[10,1]',
          'Load[11,1]','Load[12,1]','Load[13,1]','Load[14,1]','Load[15,1]',
          'Load[16,1]','Load[17,1]','Load[18,1]','Load[19,1]','Load[20,1]',
          'Load[21,1]','Load[22,1]','Load[23,1]','Load[24,1]','Load[25,1]',
          'Load[26,1]','Load[27,1]','Load[28,1]','Load[29,1]','Load[30,1]',
          'Load[31,1]','Load[32,1]','Load[33,1]','Load[34,1]','Load[35,1]',
          'Load[36,1]')),

# latent variable covariances 
mxPath(from=c('b_1i','b_2i','b_3i'), connect='unique.pairs', arrows=2,
 free=TRUE, values = c(8,0,0,.1,0,12),
 labels=c('sigma2_1','sigma_21','sigma_31','sigma2_2','sigma_32','sigma2_3')),

# latent variable means  
mxPath(from='one', to=c('b_1i','b_2i','b_3i'),
 arrows=1, free=TRUE, values=c(50, 1, -20), labels=c('beta_01','beta_02','beta_03')),

mxMatrix('Full', 36, 1, free = TRUE, values = -1, labels = 'g', name = 'gamma'),
mxMatrix('Full', 36, 1, free = FALSE, values = c( 1, 2, 3, 4, 5, 6, 7, 8, 9,10,
                                                 11,12,13,14,15,16,17,18,19,20,
                                                 21,22,23,24,25,26,27,28,29,30,
                                                 31,32,33,34,35,36), name = 'time'),
mxAlgebra(gamma*exp(gamma*time), name='Load'),

# including covariate 

mxPath(from='one', to='dur_lbr', arrows=1, free=TRUE,
 values=0, labels='omega_x1'),

# Variance
mxPath(from='dur_lbr', arrows=2, free=TRUE, values=1, labels='sigma2_x1'),

# Covariate Effects
mxPath(from='dur_lbr', to=c('b_1i','b_2i','b_3i'),
arrows=1, free=TRUE, values=0, labels=c('beta_1l','beta_12','beta_13')
),

# Indirect Effects
mxMatrix('Full', 36, 1, free = TRUE,  values = 0, labels = 'beta_12', name = 'bet_12'),
mxMatrix('Full', 36, 1, free = TRUE,  values = 0, labels = 'beta_13', name = 'bet_13'),
mxMatrix('Full', 36, 1, free = FALSE, 
    labels = c(             'Load[1,1]' ,'Load[2,1]', 'Load[3,1]', 'Load[4,1]', 'Load[5,1]', 'Load[6,1]', 'Load[7,1]', 'Load[8,1]', 'Load[9,1]',
               'Load[10,1]','Load[11,1]','Load[12,1]','Load[13,1]','Load[14,1]','Load[15,1]','Load[16,1]','Load[17,1]','Load[18,1]','Load[19,1]',
               'Load[20,1]','Load[21,1]','Load[22,1]','Load[23,1]','Load[24,1]','Load[25,1]','Load[26,1]','Load[27,1]','Load[28,1]','Load[29,1]',
               'Load[30,1]','Load[31,1]','Load[32,1]','Load[33,1]','Load[34,1]','Load[35,1]','Load[36,1]'),
   name = 'b3_load'),

mxAlgebra(bet_12+bet_13*b3_load, name='Indirect')

) # Close Model
 
jb.hght.fit <- mxRun(jb.hght.omx, intervals=TRUE)

summary(jb.hght.fit)

jb.hght.fit$Indirect

#mxAlgebra 'Indirect' 
#$formula:  bet_12 + bet_13 * b3_load 
#$result:
#              [,1]
# [1,] -0.205021658
# [2,] -0.167730389
# [3,] -0.136300992
# [4,] -0.109812031
# [5,] -0.087486911
# [6,] -0.068671112
# [7,] -0.052812997
# [8,] -0.039447645
# [9,] -0.028183213
#[10,] -0.018689454
#[11,] -0.010688035
#[12,] -0.003944371
#[13,]  0.001739246
#[14,]  0.006529447
#[15,]  0.010566668
#[16,]  0.013969272
#[17,]  0.016837015
#[18,]  0.019253974
#[19,]  0.021291007
#[20,]  0.023007835
#[21,]  0.024454793
#[22,]  0.025674301
#[23,]  0.026702113
#[24,]  0.027568361
#[25,]  0.028298442
#[26,]  0.028913761
#[27,]  0.029432357
#[28,]  0.029869434
#[29,]  0.030237806
#[30,]  0.030548273
#[31,]  0.030809938
#[32,]  0.031030471
#[33,]  0.031216338
#[34,]  0.031372988
#[35,]  0.031505014
#[36,]  0.031616287
#dimnames: NULL


mxCI("Indirect")




























